/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.reactivex.ext.bridge;

import java.util.Map;
import io.reactivex.Observable;
import io.reactivex.Flowable;
import io.reactivex.Single;
import io.reactivex.Completable;
import io.reactivex.Maybe;
import io.vertx.ext.bridge.BridgeEventType;
import io.vertx.core.json.JsonObject;
import io.vertx.core.AsyncResult;
import io.vertx.core.Handler;
import java.util.function.Function;

/**
 * Represents an event that occurs on the event bus bridge.
 * <p>
 * Please consult the documentation for a full explanation.
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.ext.bridge.BaseBridgeEvent original} non RX-ified interface using Vert.x codegen.
 */

@io.vertx.lang.rx.RxGen(io.vertx.ext.bridge.BaseBridgeEvent.class)
public class BaseBridgeEvent extends io.vertx.reactivex.core.Future<Boolean> {

  @Override
  public String toString() {
    return delegate.toString();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;
    BaseBridgeEvent that = (BaseBridgeEvent) o;
    return delegate.equals(that.delegate);
  }
  
  @Override
  public int hashCode() {
    return delegate.hashCode();
  }

  public static final io.vertx.lang.rx.TypeArg<BaseBridgeEvent> __TYPE_ARG = new io.vertx.lang.rx.TypeArg<>(    obj -> new BaseBridgeEvent((io.vertx.ext.bridge.BaseBridgeEvent) obj),
    BaseBridgeEvent::getDelegate
  );

  private final io.vertx.ext.bridge.BaseBridgeEvent delegate;
  
  public BaseBridgeEvent(io.vertx.ext.bridge.BaseBridgeEvent delegate) {
    super(delegate);
    this.delegate = delegate;
  }

  public io.vertx.ext.bridge.BaseBridgeEvent getDelegate() {
    return delegate;
  }

  /**
   * Set a handler for the result.
   * <p>
   * If the future has already been completed it will be called immediately. Otherwise it will be called when the
   * future is completed.
   * @param handler the Handler that will be called with the result
   * @return a reference to this, so it can be used fluently
   */
  public io.vertx.reactivex.core.Future<Boolean> setHandler(Handler<AsyncResult<Boolean>> handler) { 
    delegate.setHandler(handler);
    return this;
  }

  /**
   * Set a handler for the result.
   * <p>
   * If the future has already been completed it will be called immediately. Otherwise it will be called when the
   * future is completed.
   * @return a reference to this, so it can be used fluently
   */
  public Single<Boolean> rxSetHandler() { 
    return io.vertx.reactivex.impl.AsyncResultSingle.toSingle(handler -> {
      setHandler(handler);
    });
  }

  /**
   * Set the result. Any handler will be called, if there is one, and the future will be marked as completed.
   * @param result the result
   */
  public void complete(Boolean result) { 
    delegate.complete(result);
  }

  /**
   * Set the failure. Any handler will be called, if there is one, and the future will be marked as completed.
   * @param result the result
   * @return false when the future is already completed
   */
  public boolean tryComplete(Boolean result) { 
    boolean ret = delegate.tryComplete(result);
    return ret;
  }

  /**
   * The result of the operation. This will be null if the operation failed.
   * @return the result or null if the operation failed.
   */
  public Boolean result() { 
    Boolean ret = delegate.result();
    return ret;
  }

  /**
   * Compose this future with a provided <code>next</code> future.<p>
   *
   * When this (the one on which <code>compose</code> is called) future succeeds, the <code>handler</code> will be called with
   * the completed value, this handler should complete the next future.<p>
   *
   * If the <code>handler</code> throws an exception, the returned future will be failed with this exception.<p>
   *
   * When this future fails, the failure will be propagated to the <code>next</code> future and the <code>handler</code>
   * will not be called.
   * @param handler the handler
   * @param next the next future
   * @return the next future, used for chaining
   */
  public <U> io.vertx.reactivex.core.Future<U> compose(Handler<Boolean> handler, io.vertx.reactivex.core.Future<U> next) { 
    io.vertx.reactivex.core.Future<U> ret = io.vertx.reactivex.core.Future.newInstance(delegate.compose(handler, next.getDelegate()), next.__typeArg_0);
    return ret;
  }

  /**
   * Compose this future with a <code>mapper</code> function.<p>
   *
   * When this future (the one on which <code>compose</code> is called) succeeds, the <code>mapper</code> will be called with
   * the completed value and this mapper returns another future object. This returned future completion will complete
   * the future returned by this method call.<p>
   *
   * If the <code>mapper</code> throws an exception, the returned future will be failed with this exception.<p>
   *
   * When this future fails, the failure will be propagated to the returned future and the <code>mapper</code>
   * will not be called.
   * @param mapper the mapper function
   * @return the composed future
   */
  public <U> io.vertx.reactivex.core.Future<U> compose(Function<Boolean, io.vertx.reactivex.core.Future<U>> mapper) { 
    io.vertx.reactivex.core.Future<U> ret = io.vertx.reactivex.core.Future.newInstance(delegate.compose(new java.util.function.Function<java.lang.Boolean,io.vertx.core.Future<U>>() {
      public io.vertx.core.Future<U> apply(java.lang.Boolean arg) {
        io.vertx.reactivex.core.Future<U> ret = mapper.apply(arg);
        return ret.getDelegate();
      }
    }), io.vertx.lang.rx.TypeArg.unknown());
    return ret;
  }

  /**
   * Apply a <code>mapper</code> function on this future.<p>
   *
   * When this future succeeds, the <code>mapper</code> will be called with the completed value and this mapper
   * returns a value. This value will complete the future returned by this method call.<p>
   *
   * If the <code>mapper</code> throws an exception, the returned future will be failed with this exception.<p>
   *
   * When this future fails, the failure will be propagated to the returned future and the <code>mapper</code>
   * will not be called.
   * @param mapper the mapper function
   * @return the mapped future
   */
  public <U> io.vertx.reactivex.core.Future<U> map(Function<Boolean, U> mapper) { 
    io.vertx.reactivex.core.Future<U> ret = io.vertx.reactivex.core.Future.newInstance(delegate.map(mapper), io.vertx.lang.rx.TypeArg.unknown());
    return ret;
  }

  /**
   * @return an handler completing this future
   */
  @Deprecated()
  public Handler<AsyncResult<Boolean>> completer() { 
    if (cached_0 != null) {
      return cached_0;
    }
    Handler<AsyncResult<Boolean>> ret = delegate.completer();
    cached_0 = ret;
    return ret;
  }

  /**
   * Handles a failure of this Future by returning the result of another Future.
   * If the mapper fails, then the returned future will be failed with this failure.
   * @param mapper A function which takes the exception of a failure and returns a new future.
   * @return A recovered future
   */
  public io.vertx.reactivex.core.Future<Boolean> recover(Function<Throwable, io.vertx.reactivex.core.Future<Boolean>> mapper) { 
    io.vertx.reactivex.core.Future<Boolean> ret = io.vertx.reactivex.core.Future.newInstance(delegate.recover(new java.util.function.Function<java.lang.Throwable,io.vertx.core.Future<java.lang.Boolean>>() {
      public io.vertx.core.Future<java.lang.Boolean> apply(java.lang.Throwable arg) {
        io.vertx.reactivex.core.Future<Boolean> ret = mapper.apply(arg);
        return ret.getDelegate();
      }
    }), io.vertx.lang.rx.TypeArg.unknown());
    return ret;
  }

  /**
   * Apply a <code>mapper</code> function on this future.<p>
   *
   * When this future fails, the <code>mapper</code> will be called with the completed value and this mapper
   * returns a value. This value will complete the future returned by this method call.<p>
   *
   * If the <code>mapper</code> throws an exception, the returned future will be failed with this exception.<p>
   *
   * When this future succeeds, the result will be propagated to the returned future and the <code>mapper</code>
   * will not be called.
   * @param mapper the mapper function
   * @return the mapped future
   */
  public io.vertx.reactivex.core.Future<Boolean> otherwise(Function<Throwable, Boolean> mapper) { 
    io.vertx.reactivex.core.Future<Boolean> ret = io.vertx.reactivex.core.Future.newInstance(delegate.otherwise(mapper), io.vertx.lang.rx.TypeArg.unknown());
    return ret;
  }

  /**
   * Map the failure of a future to a specific <code>value</code>.<p>
   *
   * When this future fails, this <code>value</code> will complete the future returned by this method call.<p>
   *
   * When this future succeeds, the result will be propagated to the returned future.
   * @param value the value that eventually completes the mapped future
   * @return the mapped future
   */
  public io.vertx.reactivex.core.Future<Boolean> otherwise(Boolean value) { 
    io.vertx.reactivex.core.Future<Boolean> ret = io.vertx.reactivex.core.Future.newInstance(delegate.otherwise(value), io.vertx.lang.rx.TypeArg.unknown());
    return ret;
  }

  /**
   * Map the failure of a future to <code>null</code>.<p>
   *
   * This is a convenience for <code>future.otherwise((T) null)</code>.<p>
   *
   * When this future fails, the <code>null</code> value will complete the future returned by this method call.<p>
   *
   * When this future succeeds, the result will be propagated to the returned future.
   * @return the mapped future
   */
  public io.vertx.reactivex.core.Future<Boolean> otherwiseEmpty() { 
    io.vertx.reactivex.core.Future<Boolean> ret = io.vertx.reactivex.core.Future.newInstance(delegate.otherwiseEmpty(), io.vertx.lang.rx.TypeArg.unknown());
    return ret;
  }

  /**
   * @return the type of the event
   */
  public BridgeEventType type() { 
    if (cached_1 != null) {
      return cached_1;
    }
    BridgeEventType ret = delegate.type();
    cached_1 = ret;
    return ret;
  }

  /**
   * Get the raw JSON message for the event. This will be null for SOCKET_CREATED or SOCKET_CLOSED events as there is
   * no message involved. If the returned message is modified, {@link io.vertx.reactivex.ext.bridge.BaseBridgeEvent#setRawMessage} should be called with the
   * new message.
   * @return the raw JSON message for the event
   */
  public JsonObject getRawMessage() { 
    JsonObject ret = delegate.getRawMessage();
    return ret;
  }

  /**
   * Get the raw JSON message for the event. This will be null for SOCKET_CREATED or SOCKET_CLOSED events as there is
   * no message involved.
   * @param message the raw message
   * @return this reference, so it can be used fluently
   */
  public io.vertx.reactivex.ext.bridge.BaseBridgeEvent setRawMessage(JsonObject message) { 
    delegate.setRawMessage(message);
    return this;
  }

  private Handler<AsyncResult<Boolean>> cached_0;
  private BridgeEventType cached_1;

  public static  BaseBridgeEvent newInstance(io.vertx.ext.bridge.BaseBridgeEvent arg) {
    return arg != null ? new BaseBridgeEvent(arg) : null;
  }
}
