/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.reactivex.ext.web;

import java.util.Map;
import io.reactivex.Observable;
import io.reactivex.Flowable;
import io.reactivex.Single;
import io.reactivex.Completable;
import io.reactivex.Maybe;
import java.util.Map;

/**
 * Represents a browser session.
 * <p>
 * Sessions persist between HTTP requests for a single browser session. They are deleted when the browser is closed, or
 * they time-out. Session cookies are used to maintain sessions using a secure UUID.
 * <p>
 * Sessions can be used to maintain data for a browser session, e.g. a shopping basket.
 * <p>
 * The context must have first been routed to a {@link io.vertx.reactivex.ext.web.handler.SessionHandler}
 * for sessions to be available.
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.ext.web.Session original} non RX-ified interface using Vert.x codegen.
 */

@io.vertx.lang.rx.RxGen(io.vertx.ext.web.Session.class)
public class Session {

  @Override
  public String toString() {
    return delegate.toString();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;
    Session that = (Session) o;
    return delegate.equals(that.delegate);
  }
  
  @Override
  public int hashCode() {
    return delegate.hashCode();
  }

  public static final io.vertx.lang.rx.TypeArg<Session> __TYPE_ARG = new io.vertx.lang.rx.TypeArg<>(    obj -> new Session((io.vertx.ext.web.Session) obj),
    Session::getDelegate
  );

  private final io.vertx.ext.web.Session delegate;
  
  public Session(io.vertx.ext.web.Session delegate) {
    this.delegate = delegate;
  }

  public io.vertx.ext.web.Session getDelegate() {
    return delegate;
  }

  /**
   * @return The new unique ID of the session.
   */
  public io.vertx.reactivex.ext.web.Session regenerateId() { 
    io.vertx.reactivex.ext.web.Session ret = io.vertx.reactivex.ext.web.Session.newInstance(delegate.regenerateId());
    return ret;
  }

  /**
   * @return The unique ID of the session. This is generated using a random secure UUID.
   */
  public String id() { 
    String ret = delegate.id();
    return ret;
  }

  /**
   * Put some data in a session
   * @param key the key for the data
   * @param obj the data
   * @return a reference to this, so the API can be used fluently
   */
  public io.vertx.reactivex.ext.web.Session put(String key, Object obj) { 
    delegate.put(key, obj);
    return this;
  }

  /**
   * Get some data from the session
   * @param key the key of the data
   * @return the data
   */
  public <T> T get(String key) { 
    T ret = (T) delegate.get(key);
    return ret;
  }

  /**
   * Remove some data from the session
   * @param key the key of the data
   * @return the data that was there or null if none there
   */
  public <T> T remove(String key) { 
    T ret = (T) delegate.remove(key);
    return ret;
  }

  /**
   * @return true if the session has data
   */
  public boolean isEmpty() { 
    boolean ret = delegate.isEmpty();
    return ret;
  }

  /**
   * @return the time the session was last accessed
   */
  public long lastAccessed() { 
    long ret = delegate.lastAccessed();
    return ret;
  }

  /**
   * Destroy the session
   */
  public void destroy() { 
    delegate.destroy();
  }

  /**
   * @return has the session been destroyed?
   */
  public boolean isDestroyed() { 
    boolean ret = delegate.isDestroyed();
    return ret;
  }

  /**
   * @return has the session been renewed?
   */
  public boolean isRegenerated() { 
    boolean ret = delegate.isRegenerated();
    return ret;
  }

  /**
   * @return old ID if renewed
   */
  public String oldId() { 
    String ret = delegate.oldId();
    return ret;
  }

  /**
   * @return the amount of time in ms, after which the session will expire, if not accessed.
   */
  public long timeout() { 
    long ret = delegate.timeout();
    return ret;
  }

  /**
   * Mark the session as being accessed.
   */
  public void setAccessed() { 
    delegate.setAccessed();
  }

  /**
   * The short representation of the session to be added to the session cookie. By default is the session id.
   * @return short representation string.
   */
  public String value() { 
    String ret = delegate.value();
    return ret;
  }

  /**
   * @return the session data as a map
   */
  public Map<String, Object> data() { 
    Map<String, Object> ret = delegate.data();
    return ret;
  }


  public static  Session newInstance(io.vertx.ext.web.Session arg) {
    return arg != null ? new Session(arg) : null;
  }
}
