/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.reactivex.ext.web.handler;

import java.util.Map;
import io.reactivex.Observable;
import io.reactivex.Flowable;
import io.reactivex.Single;
import io.reactivex.Completable;
import io.reactivex.Maybe;
import io.vertx.core.Handler;

/**
 * A handler that serves favicons.
 * <p>
 * If no file system path is specified it will attempt to serve a resource called `favicon.ico` from the classpath.
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.ext.web.handler.FaviconHandler original} non RX-ified interface using Vert.x codegen.
 */

@io.vertx.lang.rx.RxGen(io.vertx.ext.web.handler.FaviconHandler.class)
public class FaviconHandler implements io.vertx.core.Handler<io.vertx.reactivex.ext.web.RoutingContext> {

  @Override
  public String toString() {
    return delegate.toString();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;
    FaviconHandler that = (FaviconHandler) o;
    return delegate.equals(that.delegate);
  }
  
  @Override
  public int hashCode() {
    return delegate.hashCode();
  }

  public static final io.vertx.lang.rx.TypeArg<FaviconHandler> __TYPE_ARG = new io.vertx.lang.rx.TypeArg<>(    obj -> new FaviconHandler((io.vertx.ext.web.handler.FaviconHandler) obj),
    FaviconHandler::getDelegate
  );

  private final io.vertx.ext.web.handler.FaviconHandler delegate;
  
  public FaviconHandler(io.vertx.ext.web.handler.FaviconHandler delegate) {
    this.delegate = delegate;
  }

  public io.vertx.ext.web.handler.FaviconHandler getDelegate() {
    return delegate;
  }

  /**
   * Something has happened, so handle it.
   * @param event the event to handle
   */
  public void handle(io.vertx.reactivex.ext.web.RoutingContext event) { 
    delegate.handle(event.getDelegate());
  }

  /**
   * Create a handler with defaults
   * @return the handler
   */
  public static io.vertx.reactivex.ext.web.handler.FaviconHandler create() { 
    io.vertx.reactivex.ext.web.handler.FaviconHandler ret = io.vertx.reactivex.ext.web.handler.FaviconHandler.newInstance(io.vertx.ext.web.handler.FaviconHandler.create());
    return ret;
  }

  /**
   * Create a handler attempting to load favicon file from the specified path
   * @param path the path
   * @return the handler
   */
  public static io.vertx.reactivex.ext.web.handler.FaviconHandler create(String path) { 
    io.vertx.reactivex.ext.web.handler.FaviconHandler ret = io.vertx.reactivex.ext.web.handler.FaviconHandler.newInstance(io.vertx.ext.web.handler.FaviconHandler.create(path));
    return ret;
  }

  /**
   * Create a handler attempting to load favicon file from the specified path, and with the specified max cache time
   * @param path the path
   * @param maxAgeSeconds max how long the file will be cached by browser, in seconds
   * @return the handler
   */
  public static io.vertx.reactivex.ext.web.handler.FaviconHandler create(String path, long maxAgeSeconds) { 
    io.vertx.reactivex.ext.web.handler.FaviconHandler ret = io.vertx.reactivex.ext.web.handler.FaviconHandler.newInstance(io.vertx.ext.web.handler.FaviconHandler.create(path, maxAgeSeconds));
    return ret;
  }

  /**
   * Create a handler with the specified max cache time
   * @param maxAgeSeconds max how long the file will be cached by browser, in seconds
   * @return the handler
   */
  public static io.vertx.reactivex.ext.web.handler.FaviconHandler create(long maxAgeSeconds) { 
    io.vertx.reactivex.ext.web.handler.FaviconHandler ret = io.vertx.reactivex.ext.web.handler.FaviconHandler.newInstance(io.vertx.ext.web.handler.FaviconHandler.create(maxAgeSeconds));
    return ret;
  }

  /**
   * The default max age in seconds as set in the cache-control header
   */
  public static final long DEFAULT_MAX_AGE_SECONDS = io.vertx.ext.web.handler.FaviconHandler.DEFAULT_MAX_AGE_SECONDS;

  public static  FaviconHandler newInstance(io.vertx.ext.web.handler.FaviconHandler arg) {
    return arg != null ? new FaviconHandler(arg) : null;
  }
}
