/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.reactivex.kafka.client.consumer;

import java.util.Map;
import io.reactivex.Observable;
import io.reactivex.Flowable;
import io.reactivex.Single;
import io.reactivex.Completable;
import io.reactivex.Maybe;
import java.util.List;
import org.apache.kafka.common.record.TimestampType;

/**
 * Vert.x Kafka consumer record
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.kafka.client.consumer.KafkaConsumerRecord original} non RX-ified interface using Vert.x codegen.
 */

@io.vertx.lang.rx.RxGen(io.vertx.kafka.client.consumer.KafkaConsumerRecord.class)
public class KafkaConsumerRecord<K,V> {

  @Override
  public String toString() {
    return delegate.toString();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;
    KafkaConsumerRecord that = (KafkaConsumerRecord) o;
    return delegate.equals(that.delegate);
  }
  
  @Override
  public int hashCode() {
    return delegate.hashCode();
  }

  public static final io.vertx.lang.rx.TypeArg<KafkaConsumerRecord> __TYPE_ARG = new io.vertx.lang.rx.TypeArg<>(    obj -> new KafkaConsumerRecord((io.vertx.kafka.client.consumer.KafkaConsumerRecord) obj),
    KafkaConsumerRecord::getDelegate
  );

  private final io.vertx.kafka.client.consumer.KafkaConsumerRecord<K,V> delegate;
  public final io.vertx.lang.rx.TypeArg<K> __typeArg_0;
  public final io.vertx.lang.rx.TypeArg<V> __typeArg_1;
  
  public KafkaConsumerRecord(io.vertx.kafka.client.consumer.KafkaConsumerRecord delegate) {
    this.delegate = delegate;
    this.__typeArg_0 = io.vertx.lang.rx.TypeArg.unknown();    this.__typeArg_1 = io.vertx.lang.rx.TypeArg.unknown();  }

  public KafkaConsumerRecord(io.vertx.kafka.client.consumer.KafkaConsumerRecord delegate, io.vertx.lang.rx.TypeArg<K> typeArg_0, io.vertx.lang.rx.TypeArg<V> typeArg_1) {
    this.delegate = delegate;
    this.__typeArg_0 = typeArg_0;
    this.__typeArg_1 = typeArg_1;
  }

  public io.vertx.kafka.client.consumer.KafkaConsumerRecord getDelegate() {
    return delegate;
  }

  /**
   * @return the topic this record is received from
   */
  public String topic() { 
    String ret = delegate.topic();
    return ret;
  }

  /**
   * @return the partition from which this record is received
   */
  public int partition() { 
    int ret = delegate.partition();
    return ret;
  }

  /**
   * @return the position of this record in the corresponding Kafka partition.
   */
  public long offset() { 
    long ret = delegate.offset();
    return ret;
  }

  /**
   * @return the timestamp of this record
   */
  public long timestamp() { 
    long ret = delegate.timestamp();
    return ret;
  }

  /**
   * @return the timestamp type of this record
   */
  public TimestampType timestampType() { 
    TimestampType ret = delegate.timestampType();
    return ret;
  }

  /**
   * @return the checksum (CRC32) of the record.
   */
  @Deprecated()
  public long checksum() { 
    long ret = delegate.checksum();
    return ret;
  }

  /**
   * @return the key (or null if no key is specified)
   */
  public K key() { 
    K ret = (K)__typeArg_0.wrap(delegate.key());
    return ret;
  }

  /**
   * @return the value
   */
  public V value() { 
    V ret = (V)__typeArg_1.wrap(delegate.value());
    return ret;
  }

  /**
   * @return the list of consumer record headers
   */
  public List<io.vertx.reactivex.kafka.client.producer.KafkaHeader> headers() { 
    List<io.vertx.reactivex.kafka.client.producer.KafkaHeader> ret = delegate.headers().stream().map(elt -> io.vertx.reactivex.kafka.client.producer.KafkaHeader.newInstance(elt)).collect(java.util.stream.Collectors.toList());
    return ret;
  }


  public static <K,V>KafkaConsumerRecord<K,V> newInstance(io.vertx.kafka.client.consumer.KafkaConsumerRecord arg) {
    return arg != null ? new KafkaConsumerRecord<K,V>(arg) : null;
  }

  public static <K,V>KafkaConsumerRecord<K,V> newInstance(io.vertx.kafka.client.consumer.KafkaConsumerRecord arg, io.vertx.lang.rx.TypeArg<K> __typeArg_K, io.vertx.lang.rx.TypeArg<V> __typeArg_V) {
    return arg != null ? new KafkaConsumerRecord<K,V>(arg, __typeArg_K, __typeArg_V) : null;
  }
}
